# -*- encoding: utf-8 -*-
import numpy as np
from scipy.ndimage.filters import convolve
import cv2
# ------------------


def gauss2DMatlab(shape, sigma):
    m, n = [(ss - 1.) / 2. for ss in shape]
    y, x = np.ogrid[-m:m + 1, -n:n + 1]
    h = np.exp(-(x * x + y * y) / (2. * sigma * sigma))
    h[h < np.finfo(h.dtype).eps * h.max()] = 0
    sumh = h.sum()
    if sumh != 0:
        h /= sumh
    return h


def conv2(x, y, mode='same'):
    if len(x.shape) < len(y.shape):
        dim = x.shape
        for i in range(len(x.shape), len(y.shape)):
            dim = (1,) + dim
        x = x.reshape(dim)
    elif len(y.shape) < len(x.shape):
        dim = y.shape
        for i in range(len(y.shape), len(x.shape)):
            dim = (1,) + dim
        y = y.reshape(dim)
    origin = ()
    for i in range(len(x.shape)):
        if ((x.shape[i] - y.shape[i]) % 2 == 0 and
             x.shape[i] > 1 and
             y.shape[i] > 1):
            origin = origin + (-1,)
        else:
            origin = origin + (0,)
    z = convolve(x, y, mode='constant', origin=origin)
    return z

def rotateImage(image, angle):
    image_center = tuple(np.array(image.shape)/2)
    rot_mat = cv2.getRotationMatrix2D(image_center, angle, 1.0)
    result = cv2.warpAffine(image, rot_mat, image.shape, flags=cv2.INTER_LINEAR)
    return result

def compute_real_orientation(array_I, w=17, h=17, low_pass_filter=cv2.blur,filter_size=(5,5),blur_size=(5,5)):
    row, col = array_I.shape
    array_I = array_I.astype(np.float)
    Ox = np.zeros((row,col))
    Ox[:] = 0.0
    Vx = Ox.copy()
    Vy = Vx.copy()
    Oy = Ox.copy()

    angle = Vx.copy()#array to contain all the 17*17 blocks's orientatons
    c = r = -1
    for i in xrange(0, row-h, h):
        c+=1
        for j in xrange(0, col-w, w):
            r+=1
            Dx = cv2.Sobel(array_I[i:i+h,j:j+w],cv2.CV_64F,1,0,ksize=3)#gradient component x for a 17*17block
            Dy = cv2.Sobel(array_I[i:i+h,j:j+w],cv2.CV_64F,0,1,ksize=3)#gradient component y for 17*17 block
            for k in range(0,h):
                for l in range(0,w):
                    Vy[r][c] += ((Dx[k][l])*(Dy[k][l]))**2
                    Vx[r][c] += 2*(Dx[k][l])*(Dy[k][l])
        angle[r][c] = 0.5*(np.arctan2(Vx[r][c],Vy[r][c]))#get the orientation angle for the given 16*16 block
    c = -1
    #smoothing process of the whole array angle
    row, col = angle.shape
    for i in range(0, row):
        for j in range(0, col):
            Ox[i][j] = np.cos(2*angle[i][j])
            Oy[i][j] = np.sin(2*angle[i][j])
    Ox = low_pass_filter(Ox, blur_size)
    Oy = low_pass_filter(Oy, blur_size)
    for i in range(0, row):
        for j in range(0, col):
            angle[i][j] = 0.5*np.arctan2(Oy[i][j],Ox[i][j])#take the final orientation of all 17*17 blocks
    return angle