# -*- encoding: utf-8 -*-
import numpy as np
from scipy.ndimage.filters import convolve
import cv2
# ------------------


def gauss2DMatlab(shape, sigma):
    m, n = [(ss - 1.) / 2. for ss in shape]
    y, x = np.ogrid[-m:m + 1, -n:n + 1]
    h = np.exp(-(x * x + y * y) / (2. * sigma * sigma))
    h[h < np.finfo(h.dtype).eps * h.max()] = 0
    sumh = h.sum()
    if sumh != 0:
        h /= sumh
    return h


def conv2(x, y, mode='same'):
    if len(x.shape) < len(y.shape):
        dim = x.shape
        for i in range(len(x.shape), len(y.shape)):
            dim = (1,) + dim
        x = x.reshape(dim)
    elif len(y.shape) < len(x.shape):
        dim = y.shape
        for i in range(len(y.shape), len(x.shape)):
            dim = (1,) + dim
        y = y.reshape(dim)
    origin = ()
    for i in range(len(x.shape)):
        if ((x.shape[i] - y.shape[i]) % 2 == 0 and
             x.shape[i] > 1 and
             y.shape[i] > 1):
            origin = origin + (-1,)
        else:
            origin = origin + (0,)
    z = convolve(x, y, mode='constant', origin=origin)
    return z

def rotateImage(image, angle):
    image_center = tuple(np.array(image.shape)/2)
    rot_mat = cv2.getRotationMatrix2D(image_center, angle, 1.0)
    result = cv2.warpAffine(image, rot_mat, image.shape, flags=cv2.INTER_LINEAR)
    return result

# Orientacion de la imagen


def orient(im1, GradientSigma, BlockSigma, OrientSmoothSigma):
    sze = int(np.fix(6 * GradientSigma))
    if sze % 2 == 0:
        sze += 1
    f = gauss2DMatlab((sze, sze), GradientSigma)
    (fx, fy) = np.gradient(f)
    fy = np.transpose(fy)
    Gx = conv2(im1, rotateImage(fx,-90), mode='same')
    Gy = conv2(im1, rotateImage(fy,180), mode='same')
    Gxx = Gx * Gx
    Gxy = Gx * Gy
    Gyy = Gy * Gy
    sze = int(np.fix(6 * BlockSigma))
    if sze % 2 == 0:
        sze += 1
    f = gauss2DMatlab((sze, sze), BlockSigma)
    Gxx = conv2(Gxx, f, mode='same')
    Gxy = (2) * conv2(Gxy, f, mode='same')
    Gyy = conv2(Gyy, f, mode='same')
    denom = np.sqrt(Gxy * Gxy + ((Gxx - Gyy) * (Gxx - Gyy)))
    sin2theta = Gxy / denom
    cos2theta = (Gxx - Gyy) / denom
    if OrientSmoothSigma:
        sze = int(np.fix(6 * OrientSmoothSigma))
        if sze % 2 == 0:
            sze += 1
        f = gauss2DMatlab((sze, sze), OrientSmoothSigma)
        sin2theta = conv2(sin2theta, f, mode='same')
        cos2theta = conv2(cos2theta, f, mode='same')
    orientim = (np.pi / 2) + (np.arctan2(sin2theta, cos2theta) / 2)
    #Imin = (Gyy + Gxx) / 2 - (Gxx - Gyy) * cos2theta / 2 - Gxy * sin2theta / 2
    #Imax = Gyy + Gxx - Imin
    #reliability = 1 - Imin / (Imax + 0.001)
    #coherence = ((Imax - Imin) / (Imax + Imin)) * ((Imax - Imin) / (Imax + Imin))
    #reliability = reliability * (denom > 0.001)
    return orientim