# -*- encoding: utf-8 -*-
import numpy as np
import cv2
import matplotlib.pyplot as plt
import matplotlib.cm as cm

import matplotlib.image as mpimg#Librería para guardar imagenes
# -------------


def ordfilt2(A, order):
    result = np.zeros((int(len(A)),1))
    for n in range(0, int(len(A)), 1):
        if n + order < int(len(A)):
            band = A[range(n, n + order, 1)]
            result[n, 0] = band.max()
        else:
            band = A[range(n, int(len(A)), 1)]
            result[n, 0] = band.max()
    return result


def rotateImage(image, angle):
    #Función para rotar imagen con respecto al centro de la imagen
    image_center = tuple(np.array(image.shape)/2)
    rot_mat = cv2.getRotationMatrix2D(image_center, angle, 1.0)
    result = cv2.warpAffine(image, rot_mat, image.shape, flags=cv2.INTER_LINEAR)
    return result


def freqest(im, orientim, windsze, minWaveLength, maxWaveLength):
    orientim = 2 * np.array(orientim).ravel()#ravel  reordena una matrix y la deja como un vector
    cosorient = np.mean(np.cos(orientim))
    sinorient = np.mean(np.sin(orientim))
    #mpimg.imsave("freq.png", im, cmap="gray")
    orient = np.arctan2(sinorient,cosorient) / 2 # se halla el ángulo promedio del bloque, se hace promediando los senos y cosenos
    #de los ángulos doblados antes de la reconstrucción del ángulo de nuevo
    rotim = rotateImage(im, orient/np.pi*180)#se rota la imagen para que las crestas queden verticales
    filas = len(im)
    columnas = len(im[0])
    #Se recortta la imagen rotada para que no tenga regiones inválidas
    cropsze = int(np.fix(filas / np.sqrt(2)))
    offset = int(np.fix((filas - cropsze) / 2))
    rotim = rotim[range(offset-1, offset + cropsze+1, 1), :][:, range(offset-1, offset + cropsze+1, 1)]
    mpimg.imsave("freqrot.png", rotim, cmap="gray")
    #se suman las columnas para obtener una proyeccion de los valores de los pizeles en las crestas
    proj = rotim.sum(axis=1)
    # Create a figure of size 8x6 inches, 80 dots per inch
    #plt.figure(figsize=(8, 6), dpi=80)
    # Create a new subplot from a grid of 1x1
    #plt.subplot(1, 1, 1)
    #X = np.linspace(1, len(proj), len(proj), endpoint=True)
    # Plot cosine with a blue continuous line of width 1 (pixels)
    #plt.plot(X, proj, color="blue", linewidth=1.0, linestyle="-")
    ##Se buscan picos en la projección haciendo una dilatacion y luego encontrando donde
    #la dilatación es igual a los valores originales
    dilation = ordfilt2(proj, windsze)
    maxpts = np.zeros((len(dilation), 1))
    for m in range(0, len(dilation), 1):
        if dilation[m] == proj[m] and proj[m] > np.mean(proj):
            maxpts[m] = 1
        else:
            maxpts[m] = 0
    maxind = np.nonzero(maxpts)
    #Se determina la frecuencia espacial de las crestas dividiendo la distacia entre el primer y último pico
    #sobre el número de picos -1. Si no hay picos en el bloque o si la longitud de onda es menor o mayor que el
    #rango permitido, se establece la frecuencia = 0
    if len(maxind[0]) < 2:
        freqim = 0
    else:
        NoOfPeaks = len(maxind[0])
        waveLength = float((maxind[0][NoOfPeaks-1]-maxind[0][0])/float((NoOfPeaks-1)))
        if (waveLength > minWaveLength) and (waveLength < maxWaveLength):
            freqim = float(1/waveLength)
        else:
            freqim = 0
        return freqim


def rf(im, mask, orientim, blksize, windsze, minWL, maxWL):
    #Se hace el cálculo de la frecuencia en toda la imagen, recorriendola bloque por bloque
    #Plica lo mismo que para la segmentación, no se sobrelapan los bloques, excepto al final cuando el bloque es de mayor tamaño
    #a la zona de imagen faltante
    filas, columnas = im.shape
    freq = np.zeros((len(im), len(im[0])))
    for r in range(0, filas-blksize, blksize):
        for c in range(0, columnas-blksize, blksize):
            if ((r + blksize) < filas) & ((c + blksize) < columnas):
                blkim = im[range(r, r+blksize, 1), :][:, range(c, c+blksize, 1)]
                blkor = orientim[range(r, r+blksize, 1), :][:, range(c, c+blksize, 1)]
                freq[r:r+blksize,c:c+blksize] = freqest(blkim, blkor, windsze, minWL, maxWL)
            if ((r + blksize) > filas) and ((c + blksize) > columnas):
                blkim = im[range(filas-blksize, blksize, 1), :][:, range(columnas-blksize, columnas, 1)]
                blkor = orientim[range(filas-blksize, blksize, 1), :][:, range(columnas-blksize, columnas, 1)]
                freq[filas-blksize:filas, columnas-blksize:columnas] = freqest(blkim, blkor, windsze, minWL, maxWL)
            if ((r + blksize) > filas) and ((c + blksize) < columnas):
                blkim = im[range(r, r+blksize, 1), :][:, range(columnas-blksize, columnas, 1)]
                blkor = orientim[range(r, r+blksize, 1), :][:, range(columnas-blksize, columnas, 1)]
                freq[filas - blksize:filas, columnas - blksize:columnas] = freqest(blkim, blkor, windsze, minWL, maxWL)
            if ((c + blksize) > columnas) and ((r + blksize) < filas):
                blkim = im[range(filas-blksize, blksize, 1), :][:, range(c, c+blksize, 1)]
                blkor = orientim[range(filas-blksize, blksize, 1), :][:, range(c, c+blksize, 1)]
                freq[filas - blksize:filas, columnas - blksize:columnas] = freqest(blkim, blkor, windsze, minWL, maxWL)
    freq1 = freq * mask #Elimino la frecuencia que no está en mi zona de trabajo
    x1 = np.nonzero(freq1 > 0) #Busco las frecuencias mayores a 0
    medianfreq = np.median(freq1[x1]) #Y calculo la mediana de las frecuenciasanteriormente halladas
    return freq1, medianfreq
