import numpy as np
import cv2

def build_filters():
    filters = []
    ksize = 7
    for theta in np.arange(0, np.pi, np.pi / 128):
        kern = cv2.getGaborKernel((ksize, ksize), 3.5, theta, 10.0, 0.5, 0, ktype=cv2.CV_32F)
        kern /= 1.5*kern.sum()
        filters.append(kern)
    return filters

def gabor_fn(sigma,theta,Lambda,psi,gamma):
    sigma_x = sigma;
    sigma_y = float(sigma)/gamma;

    # Bounding box
    nstds = 3;
    xmax = max(abs(nstds*sigma_x*np.cos(theta)),abs(nstds*sigma_y*np.sin(theta)));
    xmax = np.ceil(max(1,xmax));
    ymax = max(abs(nstds*sigma_x*np.sin(theta)),abs(nstds*sigma_y*np.cos(theta)));
    ymax = np.ceil(max(1,ymax));
    xmin = -xmax; ymin = -ymax;
    (x,y) = np.meshgrid(np.arange(xmin,xmax+1),np.arange(ymin,ymax+1 ));
    (y,x) = np.meshgrid(np.arange(ymin,ymax+1),np.arange(xmin,xmax+1 ));

    # Rotation
    x_theta=x*np.cos(theta)+y*np.sin(theta);
    y_theta=-x*np.sin(theta)+y*np.cos(theta);

    gb= np.exp(-.5*(x_theta**2/sigma_x**2+y_theta**2/sigma_y**2))*np.cos(2*np.pi/Lambda*x_theta+psi);
    return gb

def InvertirI(imagem):
    imagem = (255 - imagem)
    return imagem

def process(img, filters):
    filas, columnas = img.shape
    accum = np.zeros((filas,columnas))
    for kern in filters:
        fimg = cv2.filter2D(img, -1, kern)
        np.maximum(accum, fimg, accum)
    return accum

img = cv2.imread("Huella1.jpg", 0)
out = cv2.normalize(img.astype('float'), None, 0.0, 1.0, cv2.NORM_MINMAX)
filters = build_filters()
res1 = process(out, filters)
cv2.imshow('result', res1)
cv2.waitKey(0)
cv2.destroyAllWindows()
res1 = cv2.normalize(res1.astype('uint8'), None, 0, 255, cv2.NORM_MINMAX)
res1=InvertirI(res1)
thresh = 250
im_bw = cv2.threshold(res1, thresh, 255, cv2.THRESH_BINARY)[1]
cv2.imshow('result', res1)
cv2.waitKey(0)
cv2.destroyAllWindows()